; -----------------------------------
; THE new 'PRINT A CHARACTER' RESTART
; -----------------------------------
; This restart prints the character in the accumulator using the alternate
; register set so there is no requirement to save the main registers.
; There is sufficient room available to separate a space (zero) from other
; characters as leading spaces need not be considered with a space.

;; PRINT-A
L0010:
;;	AND A			; test for zero - space.
;;	JP NZ,L07F1		; jump forward if not to PRINT-CH.

;;	JP L07F5		; jump forward to PRINT-SP.

	jp L07EE		; jump forward to OUT-CH.
; ---
;
IX_to_PC
	OUT ($FD),A		; (11) Stop the NMI generator.
	JP (IX) 		; (8) forward to L0281 (after top) or L028F
; ---
	.db $09			;+ unused location. Version. PRINT PEEK 23


; ------------------------------------
; THE 'NON MASKABLE INTERRUPT' ROUTINE
; ------------------------------------
;   Jim Westwood's technical dodge using Non-Maskable Interrupts solved the
;   flicker problem of the ZX80 and gave the ZX81 a multi-tasking SLOW mode 
;   with a steady display.  Note that the AF' register is reserved for this 
;   function and its interaction with the display routines.  When counting 
;   TV lines, the NMI makes no use of the main registers.
;   The circuitry for the NMI generator is contained within the SCL (Sinclair 
;   Computer Logic) chip. 
;   ( It takes 29 clock cycles while incrementing towards zero - it was 32TS). 

;; NMI
L0066:
	EX AF,AF'		; (4) switch in the NMI's copy of the 
				;	accumulator.
	INC A			; (4) increment.
	JR Z,L006F		; (12) forward to NMI-CONT
				; when line count has incremented to zero.
;; NMI-RET
;
	EX AF,AF'		; (4)  switch out the incremented line counter
				; or test result $80
	RET 			; (10) return to User application for a while.
; ---
	.db 0,0,0		;
; ---
;   This branch is taken when the 55 (or 31) lines have been drawn.

;; NMI-CONT 
L006F:
	EX AF,AF'		; (4) restore the main accumulator.

	PUSH AF			; (11) *  Save Main Registers
	PUSH BC			; (11) **
	PUSH DE			; (11) ***
	PUSH HL			; (11) ****

;   the next set-up procedure is only really applicable when the top set of 
;   blank lines have been generated.

	LD HL,($400C)		; (16) fetch start of Display File from D_FILE
				; points to the HALT at beginning.
	SET 7,H			; (8) point to upper 32K 'echo display file'

	HALT			; (1) HALT synchronizes with NMI.  
				; Used with special hardware connected to the
				; Z80 HALT and WAIT lines to take 1 clock cycle.

; ----------------------------------------------------------------------------
;   the NMI has been generated - start counting.
;
;   The cathode ray is at the RH side of the TV.
;   First the NMI servicing, similar to CALL		=  17 clock cycles.
;   Then the time taken by the NMI for zero-to-one path =  29 cycles.(it was 32)
;   The HALT above					=  01 cycles.
;   The 3 (previously 2) instructions below		=  29 cycles.(it was 19)
;   The code at L0281 up to and including the CALL	=  43 cycles.
;   The Called routine at L02B5				=  24 cycles.
;   --------------------------------------		  ---
;   Total Z80 instructions				= 143 cycles.
;
;   Meanwhile in TV world,
;   Horizontal retrace					=  15 cycles.
;   Left blanking border 8 character positions		=  32 cycles
;   Generation of 75% scanline from the first NEWLINE   =  96 cycles
;   ---------------------------------------		   ---
;							   143 cycles
;
;   Since at the time the first JP (HL) is encountered to execute the echo
;   display another 8 character positions have to be put out, then the
;   Refresh register need to hold $F8. Working back and counteracting 
;   the fact that every instruction increments the Refresh register then
;   the value that is loaded into R needs to be $F5. :-)
;

;;	OUT ($FD),A		; (11) Stop the NMI generator.

	jp IX_to_PC		; (10) Delay

;;IX_to_PC
;;	JP (IX) 		; (8) forward to L0281 (after top) or L028F

; ---
	.db 0			;

